@@ -56,8 +56,12 @@ struct Aware {
 pub(crate) enum AwsService {
     #[clap(name = "ec2", about = "Explore EC2 resources")]
     Ec2 {
-        #[clap(long, short)]
+        #[clap(help = "Filter by VPC", long, short)]
+        #[clap(help = "List existing tags", long)]
+        list_tags: bool,
         vpc: Vec<String>,
+        #[clap(help = "Filter by tag", long, parse(try_from_str = parse_tag))]
+        tag: Vec<(String, String)>,
     },
     #[clap(name = "cf", about = "Explore CloudFormation resources")]
     CloudFormation {
@@ -86,7 +90,11 @@ async fn main() -> anyhow::Result<()> {
     };
 
     match aware.service {
-        AwsService::Ec2 { vpc } => collect_ec2(regions, vpc).await,
+        AwsService::Ec2 {
+            list_tags,
+            vpc,
+            tag,
+        } => collect_ec2(regions, list_tags, vpc, tag).await,
         AwsService::CloudFormation { stack, status } => collect_cf(regions, stack, status).await,
     }
 }
@@ -104,7 +112,12 @@ async fn get_all_regions() -> Result<Vec<String>, ec2::Error> {
     Ok(regions)
 }
 
-async fn collect_ec2(regions: Vec<String>, vpc: Vec<String>) -> anyhow::Result<()> {
+async fn collect_ec2(
+    regions: Vec<String>,
+    list_tags: bool,
+    vpc: Vec<String>,
+    tags: Vec<(String, String)>,
+) -> anyhow::Result<()> {
     let regioned_clients = regions
         .into_iter()
         .map(ec2::Region::new)
@@ -112,21 +125,26 @@ async fn collect_ec2(regions: Vec<String>, vpc: Vec<String>) -> anyhow::Result<(
 
     for region in regioned_clients {
         let shared_config = aws_config::from_env().region(region).load().await;
-        let region = format!("AWS Region {:?}", shared_config.region().id_and_name());
         let client = ec2::Client::new(&shared_config);
 
         let progress = indicatif::ProgressBar::new(1).with_style(
             indicatif::ProgressStyle::default_bar().template(
-                "[{pos}/{len} {prefix}] {msg:24} {wide_bar} [{elapsed}/{duration} ETA {eta}]",
+                "[{prefix}] {pos}/{len} | {msg:24} {wide_bar} [{elapsed}/{duration} ETA {eta}]",
             ),
         );
-        progress.set_prefix(region.clone());
-        let mut ec2 = aws::Ec2Resources::new(client);
-        progress.set_message("Collecting VPCs");
-        ec2.collect_vpcs(&vpc).await?;
-        progress.inc(1);
+        progress.set_prefix(shared_config.region().id_and_name());
+        let mut ec2 = aws::Ec2Resources::new(client, &tags);
 
-        ec2.collect(&progress).await?;
+        if list_tags {
+            progress.set_message("Collecting Tags");
+            ec2.collect_tags(&progress).await?;
+            progress.inc(1);
+        } else {
+            progress.set_message("Collecting VPCs");
+            ec2.collect_vpcs(&vpc).await?;
+            progress.inc(1);
+            ec2.collect(&progress).await?;
+        }
 
         progress.finish();
 
@@ -177,3 +195,9 @@ async fn collect_cf(
 
     Ok(())
 }
+
+fn parse_tag(text: &str) -> anyhow::Result<(String, String)> {
+    text.split_once('=')
+        .map(|(key, value)| (key.to_string(), value.to_string()))
+        .ok_or_else(|| anyhow::anyhow!("Invalid format: should be key=value"))
+}
